import xmlrpclib
import os
import Cookie 
import pprint 
import sys 
import socket
import sys
import argparse
import tempfile
import json


# load libraries from ./lib
import lib.util,lib.check,lib.run,lib.error
import model_parser
import register



def getFileExtension(filename):
       return filename[filename.find('.'):]

def convertPossibleRegister2uppaalmodel(input):
    if input.endswith('xml'):
       uppfile=""
       modeltype=model_parser.get_modeltype(input)
       if modeltype == "uppaal":
           uppfile=input
       elif modeltype == "register":
           #TODO: cleanup tmpdir afterwards
           tmpdir=tempfile.mkdtemp(prefix="sut_compare_remote")
           uppfile=tmpdir + '/' +  'uppaalfromregmodel.xml'
           regmodel=register.from_xml(input)
           regmodel.to_uppaalfile(uppfile)
       else:
           print "error: this xml input model file is not supported. Xml model file must be either an 'uppaal' or 'register' model."
           exit(1)
       return uppfile    
    else:
       return input 


def remotecompare(url,filepath1,filepath2,min_range,max_range):
    #print "========================================================"
    #print "                remote compare "
    #print "========================================================"
    #print ""

    filepath1=convertPossibleRegister2uppaalmodel(filepath1)
    filepath2=convertPossibleRegister2uppaalmodel(filepath2)

    ext1=getFileExtension(os.path.basename(filepath1)) 	
    ext2=getFileExtension(os.path.basename(filepath2)) 	
    str1=open(filepath1,'r').read()
    str2=open(filepath2,'r').read()
         

    s = xmlrpclib.ServerProxy(url)
    return s.tomte_compare(str1,ext1,str2,ext2,min_range,max_range)  
#    try:
#        return s.tomte_compare(str1,ext1,str2,ext2,min_range,max_range)  
#    except socket.error as e:
#        return e


def dump(name,data):
      print(name + ": {0}".format(pprint.pformat(data)))

def args_parsing(args):
    import os.path
    argparser = argparse.ArgumentParser(prog=os.path.basename(args[0]))

    argparser.add_argument('--keep-tmp-files', action='store_true', help='keep temporary files generated for doing comparison')
    argparser.add_argument('url', help='url of remote service, eg.  http://host.com:8000')
    argparser.add_argument('modelfile1', help='model file which can be one of : uppaal model,cadp aut, jtorx aut, dot')
    argparser.add_argument('modelfile2', help='model file which can be one of : uppaal model,cadp aut, jtorx aut, dot')
    argparser.add_argument('range_min', help='minimum value of range used for input params when flattening the model')
    argparser.add_argument('range_max', help='maximum value of range used for input params when flattening the model')

    parse_result=argparser.parse_args(args[1:])

    keeptmp=parse_result.keep_tmp_files
    url=parse_result.url

    # handle input/output file argument
    modelfile1=os.path.abspath(parse_result.modelfile1).replace('\\','/')
    modelfile2=os.path.abspath(parse_result.modelfile2).replace('\\','/')
    # Note: on windows also interprets / as dir sep, and returns path with windows \


    lib.check.check_isreadablefile(modelfile1)
    lib.check.check_isreadablefile(modelfile2)

    range_min=parse_result.range_min
    range_max=parse_result.range_max

    return  [url,modelfile1,modelfile2,range_min,range_max,keeptmp]

def main(args):
    (url,modelfile1,modelfile2,range_min,range_max,keeptmp)=args_parsing(args)
    result=remotecompare(url,modelfile1,modelfile2,range_min,range_max)
    #import pprint
    #pprint.pprint( result )

    # output result as json 
    print json.dumps(result, sort_keys=True, indent=4)

if __name__ == "__main__":
    try:
        main(sys.argv)
    except KeyboardInterrupt as e:
        print >> sys.stderr, "\n\nExiting on user cancel."
        sys.exit(1)
