#!/usr/bin/env python
"""
   generate sutinfo.yaml  file from uppaal model

   The sutinfo.yaml file describes some information about
   the model in a yaml config file. Information in this file :
    - input interfaces sut
    - output interfaces sut
    - sut name :
        By default the sut name specified in sutinfo.yaml is derived
        from the model's filename. However if the filename is model.xml
        then the name of the parent folder is taken!
    
    note: the sut name is not really needed for learning, however when 
        writing the results in an output folder it is nice if the
        output folder has the sut name in its name. Later it is always
        more clear what sut you have learned at looking at the directy's name. 

"""
# Author: Harco Kuppens


import sys, os,  pprint, re, getpass, tempfile   # standard libraries
import argparse # only in standard lib for python 2.7 and later -> installed local copy  (pure python versions)


import model_parser
import util



def args_parsing(args):

    argparser = argparse.ArgumentParser(prog=os.path.basename(args[0]))

    argparser.add_argument('modelfile', help='see below for available models')
    argparser.add_argument('sutinfofile', help='the to be generated sutinfo file ')

    str=argparser.format_help()

    def myhelp( ):
        print str 
        print """
The sutinfo.yaml file describes some information about
the model in a yaml config file. Information in this file :
- input interfaces sut
- output interfaces sut
- sut name :
    By default the sut name specified in sutinfo.yaml is derived
    from the model's filename. However if the filename is model.xml
    then the name of the parent folder is taken!

note: the sut name is not really needed for learning, however when 
    writing the results in an output folder it is nice if the
    output folder has the sut name in its name. Later it is always
    more clear what sut you have learned at looking at the directy's name. 


	"""  

    def myerror( message):
            print >>sys.stderr, ('%s: error: %s\n') % (argparser.prog, message)
            argparser.print_usage(sys.stderr)
            exit(1)

    argparser.error=myerror
    argparser.format_help=myhelp

    parse_result=argparser.parse_args(args[1:])

    # handle modelfile argument
    model_file=parse_result.modelfile
    # if model_file is not a file path to a xml file, we assume
    # only the model's name is give as input!
    if not model_file.endswith('.xml'):
       model_file= models_dir  + '/' + model_file + ".xml"
    model_file=os.path.abspath(model_file).replace('\\','/')
    # always use unix slashes in python!!
    # automatic convert any windows slashes
    # TODO: document this convention in python/txtdocs/

    # handle modelfile argument
    sutinfo_file=os.path.abspath(parse_result.sutinfofile).replace('\\','/')

    util.check_iswritabledir(os.path.dirname(sutinfo_file))

    return  [model_file,sutinfo_file]

def write_sutinfo(output_file,data):
    import yaml  # external libraries   -> installed local copy   (pure python versions) (not yaml doesn't use libyaml but pure python version)

    selected_keys=('name','inputInterfaces','outputInterfaces','constants')
    #dictionary comprehension only supported for python 2.7 and higher
    #config={k:v for k,v in data.iteritems() if k in selected_keys}
    config={}
    for k,v in data.iteritems():
        if k in selected_keys:
            config[k]=v

    constants = []
    for constant in config['constants'].values():
        constants.append(constant[1]) 
    config['constants']=constants

    with open(output_file,'w') as f:
        f.write(yaml.safe_dump(config,default_flow_style=False))

def uppaal2sutinfo(model_file,sutinfo_file):
    parsed_data=model_parser.parse_model(model_file)
    write_sutinfo(sutinfo_file,parsed_data)


def main(args):
    """
     usage: sut_uppaal2sutjavafile model sutinfofile   

     The sutinfo.yaml file describes some information about
     the model in a yaml config file. Information in this file :
       - input interfaces sut
       - output interfaces sut
       - sut name :
	    By default the sut name specified in sutinfo.yaml is derived
	    from the model's filename. However if the filename is model.xml
	    then the name of the parent folder is taken!
     
      note: the sut name is not really needed for learning, however when 
            writing the results in an output folder it is nice if the
            output folder has the sut name in its name. Later it is always
            more clear what sut you have learned at looking at the directy's name. 
    """  
    (model_file,sutinfo_file)=args_parsing(args)
    uppaal2sutinfo(model_file,sutinfo_file)


if __name__ == "__main__":
    main(sys.argv)



